"""ZERG doc_engine package for automated documentation generation."""

from __future__ import annotations

from zerg.doc_engine.detector import ComponentDetector, ComponentType

__all__ = ["ComponentDetector", "ComponentType"]
